/********************************************************************

            Pinc Dictionary Carving Functions

Author: Rod Crawford

Date: 16/11/89
Mods:  5/ 4/90 Rod Replaced enum with #defs for K&R compatability
      10/ 4/90 Rod Added CFA indirection to carve_voc for new model.
      12/ 4/90 Rod Added OS9 Support

This file contains the set of functions which carve headers, compile
colon definitions and make vocabularies.

************************************************************************/

#include "cmodel.h"
#include "vocs.h"
#include "lowram.h"
#include "nfmasks.h"
#include "vmsize.h"
#include "misc.h"



#include "enum.h"



/**************************  external data *******************************/


#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif

extern unsigned CELL dp, latest, voc_link;


/******************* External Function Prototypes  ************************/

#ifdef KNR
extern unsigned CELL comma_word();
extern void add_latest_voc_to_context();
extern unsigned CELL current_voc();
extern unsigned CELL cell_aligned();
extern unsigned CELL quotes_comma_string();
extern void strupper();
extern void strlower();
#else
extern unsigned CELL comma_word(unsigned CELL data);
extern void add_latest_voc_to_context(void );
extern unsigned CELL current_voc(void );
extern unsigned CELL cell_aligned(unsigned CELL address);
extern unsigned CELL quotes_comma_string(char *string,unsigned CELL address);
extern void strupper( char *string );
extern void strlower( char *string );
#endif

#include "extproto.h"

/**************************  Function Prototypes  ************************/

#ifdef KNR
void compile_primitives();
void compile_voc();
void compile_con();
void carve_voc();
void compile_colon();
void carve_header();
unsigned CELL thread_entry();
int hash_to_thread_number();
int hash_to_symbol_thread_number();
void smudge_latest_word();
#else
void compile_primitives(void );
void compile_voc(char *name);
void compile_con(char *name, int number);
void carve_voc(char *name);
void compile_colon(char *name,unsigned CELL opcode);
void carve_header(char *name);
unsigned CELL thread_entry(char *name);
int hash_to_thread_number(char *name);
int hash_to_symbol_thread_number(char *name, unsigned CELL max_threads);
void smudge_latest_word( void );
#endif




/**************************  compile_primitives  ********************

Desc: Compiles all the primitive words in the Pinc System.
      This function consists of a series of compile_voc, and
      compile_colon calls. These calls are generated by the FIDL
      compiler and placed in the file carve.h

**********************************************************************/

void compile_primitives( void )
{
#include "carve.h"
}




/****************************  compile_voc  ****************************
                        compile_con

Desc: Compiles a vocabulary of the given name into the dictionary.
      Since this function is called when the system is being
      initialised. It assumes that the vocabulary should be
      made immediately available within the the seach order and
      should be compiled into. It therefore adds the voc to current
      and context.

      A vocabulary has the following structure:
      1) A conventional forth name field consisiting of a thread
         link pointing back ot the previously defined word in that
         particular thread and vocabulary name.
      2) A code field containing the the Pinc primtive opcode
         do_voc.
      3) A parameter field consisiting of:
         a) The NEXT primitive to ensure return form the execution
            of a vocabulary.
         b) A voc-link pointing to the cfa of the previously defined
            vocabulary. This link used when printin a list of all
            defined vocabularys in the system and by forget.
         c) A thread table of MAX_THREADS in size.
      Note: The size of the thread table is not included as an
            entry in each forth vocabulary and remains global to
            the system.

In:   name  string containing the name of the vocabulary to be
            created.

Alg:  A Pinc header is carved inthe normal way.
      The current cfa is noted. If this is the root vocabulary i.e.
      voc_link == 0 then we store the cfa in the user variable
      ROOT_VOC. This can then be accessed by only.
      We lay down the primitives do_voc and next.
      The current value of voclink is then laid and the
      voc-link variable is updated with save cfa. Thus the voc-link
      chain is increased by one to include this vocabulary.
      The thread table is then created with all entries set to zero.
      The vocabulary is added to the front of the context array
      and made current. The is performed by the execution of the
      Pinc primitives also and definitions with the action of the
      vocabulary being mimicked by add_latest_voc_to_context.

***********************************************************************/

void compile_voc( name )
char *name;
{
    carve_voc( name );
    also();
    add_latest_voc_to_context();
    definitions();
}


void compile_con( name, number )
char *name;
int number;
{
    carve_header( name );
    comma_word( dp+CELL_SIZE );
    comma_word( (CELL) prim_do_con );
    comma_word( (signed CELL) number );
}




/****************************  carve_voc  ****************************

Desc: Carves a vocabulary of the given name into the dictionary.

      A vocabulary has the following structure:
      1) A conventional forth name field consisiting of a thread
         link pointing back ot the previously defined word in that
         particular thread and vocabulary name.
      2) A code field containing the the Pinc primtive opcode
         do_voc.
      3) A parameter field consisiting of:
         a) The NEXT primitive to ensure return form the execution
            of a vocabulary.
         b) A voc-link pointing to the cfa of the previously defined
            vocabulary. This link used when printin a list of all
            defined vocabularys in the system and by forget.
         c) A thread table of MAX_THREADS in size.
      Note: The size of the thread table is not included as an
            entry in each forth vocabulary and remains global to
            the system.

In:   name  string containing the name of the vocabulary to be
            created.

Alg:  A Pinc header is carved inthe normal way.
      The current cfa is noted. If this is the root vocabulary i.e.
      voc_link == 0 then we store the cfa in the user variable
      ROOT_VOC. This can then be accessed by only.
      We lay down the primitives do_voc and next.
      The current value of voclink is then laid and the
      voc-link variable is updated with save cfa. Thus the voc-link
      chain is increased by one to include this vocabulary.
      The thread table is then created with all entries set to zero.

***********************************************************************/

void carve_voc( name )
char *name;
{
    unsigned CELL my_cfa;
    int i;

    carve_header( name );
    comma_word( dp+CELL_SIZE );
    my_cfa = dp;
    if( !voc_link ) {
        unsigned CELL * cell;

        cell = (unsigned CELL *) &ram[ ROOT_VOC ];
        *cell = my_cfa;
    }
    comma_word( (CELL) prim_do_voc );
    comma_word( voc_link );
    voc_link = my_cfa;
    for( i=0; i<MAX_THREADS; i++ )
      comma_word( (CELL) 0 );
}






/***************************  compile_colon  ***************************

Desc: Compiles a colon defintiion for a Pinc primitive into
      The dictionary.
      Although the Pinc inner interpreter recognises a set of
      primitive instruction opcodes, with the exeception of next,
      these are never laid directly within a normal colon definition.
      The primitive opcodes a stored within a conventional forth word
      structure created by this function. This may compile primtives
      such as dup and drop as well as execute them from the keyboard.
      An alternative to this approach would have been to make a
      separate child of a defining word for each primtive and
      have this either compile the primitive opcode or execute
      it.

In:   name    the string containing the name of the word to be compiled.
      opcode  the opcode to be associated with the name supplied.

Alg:  A header of the supplied name is carved in the dictionary.
      This followed by the primitive opcode which is stored in the
      cfa space and the opcode next which is stored in the pfa.

***********************************************************************/

void compile_colon( name, opcode )
char name[];
unsigned CELL opcode;
{
    carve_header( name );
    comma_word( opcode );
    comma_word( (CELL) prim_next );
}



/*************************  carve_header  **************************

Desc: Carves a header of the supplied name in the dictionary.

In:   name  string containing the name of the header to be
            carved.

Alg:  If a vocabulary has been previously defined
         i.e. voc-link != 0
      then
          move the dp to the next free whole cell
          find a thread number by hashing on name
          store entry in thread table at ram(dp)
          and ram(dp) in thread table thus
          adding to the thread linked list.
      else
          this must be the first voc to be compiled
        so lay zero for thread link.
      convert the name string to the required case.
      set latest to nfa and lay the name string down
      Or in the name field identifier bit to enable
      easy traversal from the cfa to the nfa.
      This bit must be higher than any character in a forth
      name string e.g. bit 7.

*******************************************************************/

void carve_header( char name[]) {

    unsigned CELL *here_cell, *thread_cell, thread;

    if( voc_link ) {
      dp = cell_aligned( dp );
      thread = thread_entry( name );
      here_cell = (unsigned CELL *) &ram[ dp ];
      thread_cell = (unsigned CELL *) &ram[ thread ];
      *here_cell = *thread_cell;
      *thread_cell = dp;
      dp += CELL_SIZE;

    } else {
      comma_word( (CELL) 0 );
    }

#ifdef NO_CASE_SENSITIVITY
#warning "NO_CASE_SENSITIVITY"

#ifdef USE_LOWER_CASE
#warning "USE_LOWER_CASE"
      strlower( name );
#else
#warning "USE_UPPER_CASE"
      strupper( name );
#endif
#else
#ifdef USE_LOWER_CASE
#if IMPLEMENTATION != SUN
      #error You must be case insensitive to use lower case for Pinc
#endif
#endif
#endif

    latest = dp;
    dp = quotes_comma_string( name, dp );
    ram[ latest ] |= NFA_MASK;
}




/*****************************  thread_entry  *****************************

Desc: Given a string representing a word name, this function returns
      the address of a thread in the current vocabulary which is
      associated with that name.


In:   name  name of the word to be hashed into a thread address.

Ret:  address of the thread in the current voacabulary (but not
      relative to it) which is associated with that name.

Alg:  The thread_entry is calculated using a hashing algorithm:
      has_to_thread_number. This function returns an index into
      a thread table for the name supplied. This is then added
      to the address of the the current vocabularies thread table.
      The address of the thread table is given by the equation:

      current_voc + 2*CELL_SIZE

      2*CELL_SIZE is used to step over the prim_voc and voc-link
      elements of a vocabulary.

**************************************************************************/

unsigned CELL thread_entry( name )
char *name;
{
    unsigned CELL voc, thread;

    voc = current_voc();
    thread = voc + (2*CELL_SIZE) + (CELL_SIZE*hash_to_thread_number( name ));

    return( thread );
}



/************************  hash_to_thread_number  *********************

Desc: Returns an index into a thread table for a given name.

In:   name   string representing the name to be hashed.

Ret:  index into a thread table

Alg:  The hashing algorithm takes the first character of the name
      adds it to the length of the name and take the modulus of
      MAX_THREADS. This is achieved by anding with MAX_THREADS-1.
      This is more efficient for obvious reasons.

***********************************************************************/

int hash_to_thread_number( name )
char *name;
{
    return( ((int) name[0] + strlen( name )) & (MAX_THREADS-1) );
}




/*********************hash_to_symbol_thread_number  *********************

Desc: Returns an index into a thread table for a given name.

In:   name    string representing the name to be hashed.
      max_threads  maximum number of threads in thread table

Ret:  index into a thread table

Alg:  The hashing algorithm takes the first character of the name
      adds it to the length of the name and take the modulus of
      MAX_THREADS. This is achieved by anding with MAX_THREADS-1.
      This is more efficient for obvious reasons.


***********************************************************************/

int hash_to_symbol_thread_number( name, max_threads )
char *name;
unsigned CELL max_threads;
{
    return( ((int) name[0] + strlen( name )) & (int) (max_threads-1) );
}




/*******************  smudge_latest_word  *****************************

Desc: Smudges the last word defined.

********************************************************************/

void smudge_latest_word( void )
{
    ram[ latest ] ^= SMUDGE_MASK;
}
