/********************************************************************

            Pinc Dictionary Storage Words

Author: Rod Crawford

Date: 15/11/89
Mods:  9/ 4/90 Rod Changed LIT to compile the actual token value
             not lits address in the diciotnary.
      12/ 4/90 Rod Added OS9 Support

The following words manipulated the dictionary pointer and store
data into ram.

*********************************************************************/


#include "cmodel.h"
#include "pinc.h"
#include "vmsize.h"
#include "enum.h"




/**************************  external data *******************************/


#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif


extern unsigned CELL dp;



/******************* External Function Prototypes  ************************/

#ifdef KNR
extern unsigned CELL nfa_to_lfa();
extern unsigned CELL lfa_to_nfa();
extern unsigned CELL nfa_to_cfa();
extern unsigned CELL cfa_to_nfa();
extern unsigned CELL defined();
extern unsigned CELL execution_token();
#else
extern unsigned CELL nfa_to_lfa(unsigned CELL address);
extern unsigned CELL lfa_to_nfa(unsigned CELL address);
extern unsigned CELL nfa_to_cfa(unsigned CELL nfa);
extern unsigned CELL cfa_to_nfa(unsigned CELL cfa);
extern unsigned CELL defined(char *word_name);
extern unsigned CELL execution_token(unsigned CELL lfa);
#endif



/**************************  Function Prototypes  ************************

#ifdef KNR
unsigned CELL cell_aligned();
unsigned CELL quotes_comma_string();
unsigned CELL comma_word();
unsigned CELL comma_byte();
void compile_literal();
#else
unsigned CELL cell_aligned(unsigned CELL address);
unsigned CELL quotes_comma_string(char *string,unsigned CELL address);
unsigned CELL comma_word(unsigned CELL data);
unsigned CELL comma_byte(unsigned char data);
void compile_literal(unsigned CELL value);
#endif



/**********************  cell_aligned  ***************************

Desc: If the given address is not on a cell boundary this
      function returns the the address of the next cell above the
      given address in ram.
      Otherwise if the given address is on a cell boundary that
      value is returned.

In:   address address to be checked

Ret:  cell aligned address

****************************************************************/

unsigned CELL cell_aligned( address )
unsigned CELL address;
{
    return( (address % CELL_SIZE) ?
          address + CELL_SIZE - (address % CELL_SIZE) :
          address );
}



/********************  quotes_comma_string  *************************

Desc: Stores the given string in ram at the given address
      The address of the next cell after the string is returned.
      Note: string are currently stored with a count byte and
      a null. This may be chaneged to just null terminated in
      later versions.

In:   string  string to be stored in ram
      address  address at which the string is to be stored

Ret:  address of next free cell after stored string.

********************************************************************/

unsigned CELL quotes_comma_string( string, address )
char string[];
unsigned CELL address;
{
    ram[ address++ ] = (unsigned char) ( strlen( string ) );
    strcpy( &ram[ address ], string );
    return( cell_aligned( address + strlen( string ) + 1 ));
}



/************************  comma_word  ******************************

Desc: Stores the given word (CELL) of data at the address pointed
      to by the dictionary pointer. The dictionary is moved on to
      point at the next cell.

In:   data   cell of data to be stored.

Ret:  new value of the dictionary pointer.

*********************************************************************/

unsigned CELL comma_word( data )
unsigned CELL data;
{
    unsigned CELL * cell;

    cell = (unsigned CELL *) &ram[ dp ];
    *cell = data;
    dp += CELL_SIZE;
    return( dp );
}



/****************************  comma_byte  **************************

Desc: Stores the given byte (CELL) of data at the address pointed
      to by the dictionary pointer. The dictionary is moved on to
      point at the next byte.

In:   data   byte of data to be stored.

Ret:  new value of the dictionary pointer.

*********************************************************************/


unsigned CELL comma_byte( data )
unsigned char data;
{
    ram[ dp++ ] = data;
    return( dp );
}



/*************************  compile_literal ***************************

Desc: Compiles a literal into the dictionary. This consists of the
      execution toek for the word lit followed by the value of the literal.

In:   value  numeric value of the literal.

**********************************************************************/

void compile_literal( value )
unsigned CELL value;
{
    comma_word( (CELL) prim_lit );
    comma_word( value );
}
