/***********************************************************************

        Pinc Number to String Conversion Functions

Author: Rod Crawford.

Date: 15/11/89
Mods:  5/ 4/90 Rod Added small model support for print_one_stack_line
      12/ 4/90 Rod Added support for OS9

This file contains a set of word for converting number into strings.
In addition the function print_one_stack line is provided for
use by .s

************************************************************************/

#include "cmodel.h"
#include "pinc.h"
#include "lowram.h"
#include "vmsize.h"
#include "strsizes.h"




/**************************  external data *******************************/


#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif

extern struct from_fileblock {
     char filename[ FROM_FILENAME_LENGTH ];
     int page_number, line_number, line_in_page_number,
         last_page_required;
     FILE *stream;
       };

extern struct from_fileblock output;



/*************************  Function Prototypes  ***************************/

#ifdef KNR
void number_to_string();
void unsigned_number_to_string();
void store_digits();
void unsigned_store_digits();
void swap_string();
void print_string_in_field();
void convert_to_standard_bases();
void print_one_stack_line();
#else
void number_to_string(CELL number,char *string);
void unsigned_number_to_string(unsigned CELL number,char *string);
void store_digits(CELL number,char *string,int *end_ptr);
void unsigned_store_digits(unsigned CELL number,char *string,int *end_ptr);
void swap_string(char *string,int back_ptr);
void print_string_in_field(char *string,unsigned CELL field_width,char leader_char);
void convert_to_standard_bases(unsigned CELL number,char *in_decimal,char *in_hex,char
*in_binary);
void print_one_stack_line(unsigned CELL current_base,char *in_decimal,char *in_hex,char
*in_binary);
#endif



/**********************  number_to_string  ******************************

Desc: Converts the given signed number to a string of digits

In:   number  number to be converted

Out:  string  string of characters representing the number.

************************************************************************/

void number_to_string( number, string )
CELL number;
char string[];
{
    int end_ptr;

    store_digits( number, string, &end_ptr );
    if( number < (CELL) 0 )
        string[ ++end_ptr ] = '-';
    swap_string( string, end_ptr );
}



/*******************  unsigned_number_to_string  ************************

Desc: Converts the given unsigned number to a string of digits

In:   number  number to be converted

Out:  string  string of characters representing the number.

************************************************************************/

void unsigned_number_to_string( number, string )
unsigned CELL number;
char string[];
{
    int end_ptr;

    unsigned_store_digits( number, string, &end_ptr );
    swap_string( string, end_ptr );
}



/*************************  store_digitis  *************************

Desc: Store the character digits of the given signed number in a
      string in reverse order.

In: number  the number to be stored

Out: string   the string representing the number in reverse order.
     end_ptr  pointer to the last character in the string.


******************************************************************/

void store_digits( number, string, end_ptr )
CELL number;
char string[];
int *end_ptr;
{
    int digit;
    unsigned CELL base, *cell;

    cell = (unsigned CELL *) &ram[ BASE ];
    base = *cell;

#ifdef CELL_IS_LONG_LONG
    number = llabs( number );
#else
#ifdef CELL_IS_LONG
    number = labs( number );
#else
    number = abs( number );
#endif
#endif

    *end_ptr = 0;
    string[ *end_ptr ] = '\0';
    do {
          digit = (int) (number % base);
          string[ ++(*end_ptr) ] = (digit < 10
                                          ? (char) (digit + (int) '0')
                           : (char) (digit + (int) 'a'-10));
     number = number / base;
    }
    while( number != (CELL) 0 );
}



/*******************  unsigned_store_digitis  *********************

Desc: Store the character digits of the given unsigned number in a
      string in reverse order.

In: number  the number to be stored

Out: string   the string representing the number in reverse order.
     end_ptr  pointer to the last character in the string.


******************************************************************/

void unsigned_store_digits( number, string, end_ptr )
unsigned CELL number;
char string[];
int *end_ptr;
{
    unsigned int digit;
    unsigned CELL base, *cell;

    cell = (unsigned CELL *) &ram[ BASE ];
    base = *cell;

    *end_ptr = 0;
    string[ *end_ptr ] = '\0';
    do {
          digit = (unsigned int) (number % base);
          string[ ++(*end_ptr) ] = (digit < 10
                                          ? (char) (digit + (int) '0')
                                          : (char) (digit + (int) 'a'-10));
          number = number / base;
    }
    while( number != (CELL) 0 );
}



/*************************  swap_string  *************************

Desc: Reverses a given string.

In:    back_ptr  pointer to the last character in the string

InOut: string    string which is reversed

*******************************************************************/

void swap_string( string, back_ptr )
char string[];
int back_ptr;
{
    int front_ptr;
    char this_char;

    for( front_ptr = 0; front_ptr < back_ptr; front_ptr++ ) {
        this_char = string[ back_ptr ];
        string[ back_ptr-- ] = string[ front_ptr ];
        string[ front_ptr ] = this_char;
   }
}


/***********************  print_string_in_field  ***********************

Desc: Prints the given string in the given field width. If the string is
      shorter than the given field width the field is padded out with
      leading spaces.
      If it exceeds the field width the whole string is printed out
      anyway.

In:  string       string to be printed out.
     field_width  width of the field in which the string is to be printed

***********************************************************************/

void print_string_in_field( string, field_width, leader_char )
char string[], leader_char;
unsigned CELL field_width;
{
    int i;
    unsigned CELL *cell;

    for( i=strlen( string ); i<field_width; i++ )
     putc( leader_char, output.stream );

    fprintf( output.stream, "%s", string );
    cell = (unsigned CELL *) &ram[ OUT ];
    (*cell) += field_width;
}



/******************  convert_to_standard_bases  **********************

Desc: Converts given number to three strings representing the
      number in decimal hex and binary.
      The current value of base is lost should be preserved by
      the calling routine.

In:   number      the number to be converted.

Out:  in_decimal  the number as a decimal string.
      in_hex      the number as a hex string.
      in_binary   the number as a binary string.

*************************************************************************/

void convert_to_standard_bases( number, in_decimal, in_hex, in_binary )
unsigned CELL number;
char in_decimal[], in_hex[], in_binary[];
{
    unsigned CELL *cell;

    cell = (unsigned CELL *) &ram[ BASE ];
    *cell = 10;
//    unsigned_number_to_string( number, in_decimal );
    number_to_string( number, in_decimal );
    *cell = 16;
    unsigned_number_to_string( number, in_hex );
    *cell = 2;
    unsigned_number_to_string( number, in_binary );
}



/***************************  print_one_stack_line  **********************

Desc: Prints one line of stack. Used by .S

In: current_base  current base we are workin in.
    in_decimal    string representing number in decimal
    in_hex        string representing number in hex
    in_binary     string representing number in binary

*************************************************************************/
#ifdef CELL_IS_LONG
void print_one_stack_line( current_base, in_decimal, in_hex, in_binary )
unsigned CELL current_base;
char in_decimal[], in_hex[], in_binary[];
{
     if( current_base == 16 ) {
         print_string_in_field( in_hex, (CELL) 9, ' ' );
         fprintf( output.stream, "   " );

         print_string_in_field( in_decimal, (CELL) 11, '0' );
         fprintf( output.stream, "-d   " );

         print_string_in_field( in_binary, (CELL) 32, '0' );
         fprintf( output.stream, "-b\n" );
        }
        else
            if( current_base == 2 ) {
                print_string_in_field( in_binary, (CELL) 32, ' ' );
                fprintf( output.stream, "   " );

                print_string_in_field( in_hex, (CELL) 8, '0' );
                fprintf( output.stream, "-h   " );

                print_string_in_field( in_decimal, (CELL) 11, '0' );
                fprintf( output.stream, "-d\n" );
            }
            else {
                print_string_in_field( in_decimal, (CELL) 11, ' ' );
                fprintf( output.stream, "   " );

                print_string_in_field( in_hex, (CELL) 8, '0' );
                fprintf( output.stream, "-h   " );

                print_string_in_field( in_binary, (CELL) 32, '0' );
                fprintf( output.stream, "-b\n" );
            }
}
#else
void print_one_stack_line( current_base, in_decimal, in_hex, in_binary )
unsigned CELL current_base;
char in_decimal[], in_hex[], in_binary[];
{
     if( current_base == 16 ) {

         print_string_in_field( in_hex, (CELL) 5, ' ' );
         fprintf( output.stream, "   " );

         print_string_in_field( in_decimal, (CELL) 6, '0' );
         fprintf( output.stream, "-d   " );

         print_string_in_field( in_binary, (CELL) 16, '0' );
         fprintf( output.stream, "-b\n" );
        }
        else
            if( current_base == 2 ) {
          print_string_in_field( in_binary, (CELL) 16, ' ' );
          fprintf( output.stream, "   " );

          print_string_in_field( in_hex, (CELL) 4, '0' );
          fprintf( output.stream, "-h   " );

          print_string_in_field( in_decimal, (CELL) 6, '0' );
          fprintf( output.stream, "-d\n" );
            }
            else {
          print_string_in_field( in_decimal, (CELL) 6, ' ' );
          fprintf( output.stream, "   " );

          print_string_in_field( in_hex, (CELL) 4, '0' );
          fprintf( output.stream, "-h   " );

          print_string_in_field( in_binary, (CELL) 16, '0' );
          fprintf( output.stream, "-b\n" );
            }
}

#endif
