/**********************************************************************

        Pinc Primitives

Author: Rod Crawford

Date: 16/11/89

Mods: 12/ 4/90 Rod Added support for OS9

 The functions which define the Pinc primitives are inserted here.
 These are generated by the FIDL compiler and can be found in the file
 prim.h. In addition the the function assign_the_primitives assignes their
 addresses into the execution vector.

**********************************************************************/

#include "cmodel.h"
#include "pinc.h"
#include "misc.h"
#include "vmsize.h"
#include "lowram.h"
#include "vocs.h"
#include "strsizes.h"
#include "struct.h"
#include "nfmasks.h"
#include "os.h"

#if IMPLEMENTATION == MicrosoftC
#include <dos.h>
#endif

#if IMPLEMENTATION != MinGW64
  #include <sys/utsname.h>
#endif

#if IMPLEMENTATION == MinGW64
  #include <conio.h>
  #include <stdio.h>
#endif

#include <limits.h>
#include "ath.h"

/**************************  external data *******************************/


extern unsigned CELL data_stack[ DATA_STACK_TOP ];
extern unsigned CELL return_stack[ RETURN_STACK_TOP ];
extern char error_filename[ ERROR_FILENAME_LENGTH ];

#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif


extern unsigned CELL sp, rp, dp, ip, latest, voc_link;
extern unsigned CELL compiling;
#ifdef CWEED
extern int kill_switch;
#endif
extern unsigned CELL current_sp;
extern jmp_buf environment;
extern struct from_fileblock {
     char filename[ FROM_FILENAME_LENGTH ];
     int page_number, line_number, line_in_page_number,
         last_page_required;
     FILE *stream;
       };

extern struct from_fileblock input, output;



#include "enum.h"

typedef void (*function_pointer) (void);
extern function_pointer execution_vector[ prim_null_primitive ];



/*********************  External Functions ******************************/

extern void test_dongle();

#ifdef KNR
extern void assign_the_primitives();
extern int get_word();
extern unsigned CELL defined();
extern void carve_voc();
extern void do_error();
extern void show_this_error();

#ifdef CHECK_STACKS
extern void push_data();
extern unsigned CELL pop_data();
extern void push_return();
extern unsigned CELL pop_return();
#else
#include "stacks.h"
#endif

extern void number_to_string();
extern void unsigned_number_to_string();
extern void convert_to_standard_bases();
extern void print_string_in_field();
extern void print_one_stack_line();
extern unsigned CELL comma_word();
extern unsigned CELL nfa_to_lfa();
extern unsigned CELL lfa_to_nfa();
extern unsigned CELL nfa_to_cfa();
extern unsigned CELL cfa_to_nfa();
extern void push_from_fileblock();
extern struct from_fileblock pop_from_fileblock();
extern void init_from_stack();
extern void trash_rest_of_line();
extern int get_string();
extern unsigned CELL quotes_comma_string();
extern void compile_literal();
extern void prompt();
extern void set_raw_io();
extern void set_cooked_io();
extern int key_pressed();
extern int compile_word();
extern int number();
extern unsigned CELL comma_byte();
extern unsigned CELL cell_aligned();
extern int get_line();
extern void smudge_latest_word();
extern void forget_me();
extern void strupper();
extern void strlower();
extern int hash_to_thread_number();
extern unsigned CELL defined_in_this_thread();
extern unsigned CELL defined_in_this_voc();
extern unsigned CELL defined_in_xc_vocabulary();
extern int is_immediate();
extern unsigned CELL execution_token();
extern struct from_fileblock make_from_fileblock();
extern void goto_page();

#else
extern void assign_the_primitives(void );
extern int get_word(char *word);
extern unsigned CELL defined(char *word_name);
extern void carve_voc(char *name);
extern void do_error(unsigned CELL error_number);
extern void show_this_error( CELL error_number );
#ifdef CHECK_STACKS
extern void push_data(unsigned CELL data);
extern unsigned CELL pop_data(void );
extern void push_return(unsigned CELL address);
extern unsigned CELL pop_return(void );
#else
#include "stacks.h"
#endif

extern void number_to_string(CELL number,char *string);
extern void unsigned_number_to_string(unsigned CELL number,char *string);
extern void convert_to_standard_bases(unsigned CELL number,char *in_decimal,
                                      char *in_hex,char *in_binary);
extern void print_string_in_field(char *string,unsigned CELL field_width,
                                  char leader_char);
extern void print_one_stack_line(unsigned CELL current_base,
                                 char *in_decimal,char *in_hex,char *in_binary);
extern unsigned CELL comma_word(unsigned CELL data);
extern unsigned CELL nfa_to_lfa(unsigned CELL address);
extern unsigned CELL lfa_to_nfa(unsigned CELL address);
extern unsigned CELL nfa_to_cfa(unsigned CELL nfa);
extern unsigned CELL cfa_to_nfa(unsigned CELL cfa);
extern void push_from_fileblock(struct from_fileblock);
extern struct from_fileblock pop_from_fileblock();
extern void init_from_stack(void );
extern void trash_rest_of_line(void );
extern int get_string(char *string,char delimiter);
extern unsigned CELL quotes_comma_string(char *string,unsigned CELL address);
extern void compile_literal(unsigned CELL value);
extern void prompt(void );
extern int compile_word(char *word_name);
extern int number(char *string,int string_ptr,unsigned CELL *value);
extern unsigned CELL comma_byte(unsigned char data);
extern unsigned CELL cell_aligned(unsigned CELL address);
extern int get_line(void );
extern void smudge_latest_word( void );
extern void forget_me(unsigned CELL nfa);
extern void strupper( char *string );
extern void strlower( char *string );
extern int hash_to_thread_number(char *name);
extern unsigned CELL defined_in_this_thread(char *word_name, int name_length,
                             unsigned CELL *thread_link);
extern unsigned CELL defined_in_this_voc( char *word_name, unsigned CELL voc );
extern unsigned CELL defined_in_xc_vocabulary( char *word_name,
                                unsigned CELL thread_table,
                                unsigned CELL symbol_table_base );

extern int is_immediate(unsigned CELL nfa);
extern unsigned CELL execution_token(unsigned CELL lfa);
extern struct from_fileblock make_from_fileblock(char *filename, FILE *stream);
extern void goto_page( unsigned CELL page_number );

#endif


#include "proto.h"
#if IMPLEMENTATION == RISCOS
#include "prim1.h"
#else
#include "prim.h"
#endif


/*********************assign_the_primitives  *****************************

Desc: Assigns all Pinc primitives into the execution vector.
      The include file loaded below was generated by the FIDL
      compiler.

***********************************************************************/

void assign_the_primitives( void )
{
#include "assign.h"
}
