/*******************************************************************

                Pinc Stack Operations

Author: Rod Crawford.

Date: 15/11/89

Mods: 12/ 4/90 Rod Added support for OS9.

The following functions define the basic operation that can be perfomed
upon the data and return stacks. At present these functions are used
extensively by the Pinc primitives and no manipulation of the array
strucutures representing the stacks is allowed except through these
functions. Thus the code for the Pinc primitves is specfication of the
stack manipulation involded but is not efficient in terms of execution.
Later version of Pinc may define primitives to explicitly maniplutate
the arrays and registers which define the stacks.
By use of a #inline compiler directive the stack manipulation functions
may be inserted inline inside Pinc primitves. This will allow optimising
compilers to remove redundant variable assignments and manipulations.

************************************************************************/

#include "cmodel.h"
#include "vmsize.h"
#include "misc.h"
#undef CHECK_STACKS

/**************************  external data *******************************/


extern unsigned CELL data_stack[ DATA_STACK_TOP ];
extern unsigned CELL return_stack[ RETURN_STACK_TOP ];
#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif


extern unsigned CELL sp, rp;



/******************* External Function Prototypes  ************************/

#ifdef KNR
extern void do_error();
#else
extern void do_error(unsigned CELL error_number);
#endif



/************************  Function Prototypes  *************************/

#ifdef KNR
void push_data();
unsigned CELL pop_data();
void push_return();
unsigned CELL pop_return();
#else
void push_data(unsigned CELL data);
unsigned CELL pop_data(void );
void push_return(unsigned CELL address);
unsigned CELL pop_return(void );
#endif



/**************************  push_data  ********************************

Desc: Pushes One CELL of data onto the data stack. The stack pointer
      is pre-decremented to point where the data is placed.

In: Data  The data item to be pushed on the stack.

***********************************************************************/

void push_data( data )
unsigned CELL data;
{
    data_stack[ ++sp ] = data;
#ifdef CHECK_STACKS
    if( !(( sp >=0 && sp<DATA_STACK_TOP ) || sp==-1 ))
     do_error( (CELL) 6 );
#endif
}




/************************  pop_data  **********************************

Desc: Pops CELL from the top of data the stack and returns it.
      The stack pointer is post incremented to point at the next item
      in the stack.

Ret:  Data item popped from the stack.

**********************************************************************/

unsigned CELL pop_data( void )
{
    CELL data;

    data = data_stack[ sp-- ];
#ifdef CHECK_STACKS
    if( !(( sp >=0 && sp<DATA_STACK_TOP ) || sp==-1 ))
     do_error( (CELL) 1 );
#endif
    return( data );
}




/**************************  push_return  ********************************

Desc: Pushes One address onto the return stack. The stack pointer
      is pre-decremented to point where the return is placed.

Args: Address  The item to be pushed onto the stack.

***********************************************************************/

void push_return( address )
unsigned CELL address;
{
    return_stack[ ++rp ] = address;
#ifdef CHECK_STACKS
    if( !( rp >=0 && rp<RETURN_STACK_TOP || rp == -1 ))
     do_error( (CELL) 156 );
#endif
}




/************************  pop_return  **********************************

Desc: Pops CELL from the top of return the stack and returns it.
      The stack pointer is post incremented to point at the next item
      in the stack.

Ret:  Item popped from the stack.

**********************************************************************/

unsigned CELL pop_return( void )
{
    CELL address;

    address = return_stack[ rp-- ];

#ifdef CHECK_STACKS
    if( !( rp >=0 && rp<RETURN_STACK_TOP || rp == -1 ))
     do_error( (CELL) 157 );
#endif
    return( address );
}
