/*******************************************************************

    Pinc String to Number Conversion Functions.

Author: Rod Crawford.

Date:	6/12/89

Mods:	12/ 4/90 Rod Added support for OS9

This file contains the functions used by Pinc/Cweed Forth
to convert a string into a number depending on the base.

********************************************************************/


#include "cmodel.h"
#include "pinc.h"
#include "vmsize.h"
#include "lowram.h"
#include "strsizes.h"




/**************************  external data *******************************/


#if IMPLEMENTATION == MicrosoftC
    extern unsigned char huge ram[ RAM_TOP ];
#else
#if IMPLEMENTATION == OS9
    extern remote unsigned char ram[ RAM_TOP ];
#else
    extern unsigned char ram[ RAM_TOP ];
#endif
#endif





/******************* External Function Prototypes  ************************/

#ifdef KNR
extern void strupper();
extern void strlower();
#else
extern void strupper( char *string );
extern void strlower( char *string );
#endif





/*************************  Function Prototypes ********************/

#ifdef KNR
int number();
int digit();
#else
int number(char *string,int string_ptr,unsigned CELL *value);
int digit(char ch);
#endif



/****************************  number  ******************************

Desc: CWeed Forth Version of number?

In:   string      string possibly representing number
      string_ptr  pointer to were scan for number begins.

Out:  value       pointer to varaible which will hold the nunber.

Ret:  Returns true if conversion was successful else false.

*****************************************************************/

int number( input_string, string_ptr, value )
char input_string[];
int string_ptr;
unsigned CELL * value;
{
    int was_number;
    unsigned CELL base, *cell;
    char string[ LINE_LENGTH ], *result;

    cell = (unsigned CELL *) &ram[ BASE ];
    base = *cell;

    result = strcpy( string, input_string );
    strlower( string );
    if( string[ string_ptr ] == '-' ) {
        was_number = number( string, string_ptr+1, value );
        *value = *value * -1;
    }
    else
        if( string[ string_ptr ] == '+' )
            was_number = number( string, string_ptr+1, value );
        else
        {
            *value = 0;
            while( digit( string[ string_ptr ])) {
                *value = (*value * base) + ((CELL) string[ string_ptr ])
                          - (CELL) (isdigit( string[ string_ptr ] ) ?
                                          '0' : ((int)'a')-10);
                string_ptr++;
            }
            was_number = string[ string_ptr ] == '\0';
         }
    return( was_number );
}



/*******************************  digit  *****************************

Desc: Returns true if supplied character is valid digit.
      The digit range is defined by the user variable BASE.

In:   ch   character to be checked

Ret:  Returns True if ch is within the valid digit range, otherwise
      False.

*********************************************************************/

int digit( ch )
char ch;
{
    unsigned CELL base, *cell;

    cell = (unsigned CELL *) &ram[ BASE ];
    base = *cell;
    return( ( (base<=10) && (ch>='0') && (ch< (char) (base+ (int) '0') ))||
              ( (base>10)  && ( ((ch>='0') && (ch<='9') ) ||
                                ((ch>='a') &&
                                 (ch< (char)(base-10+(int)'a')) )))   );
}
